/**
 * Author: Hendrik van Arragon, 2023
 * SPDX-License-Identifier: MIT
 */

#include "http.h"

#include <filesystem>
#include <fstream>
#include <iostream>
#include <abmt/os.h>
#include <abmt/util/str_utils.h>

using namespace std;
using namespace io;

namespace fs = std::filesystem;

string read_file(std::string filename) {
	abmt::die_if(fs::exists(filename) == false, "Unable to open file: " + filename);
	ifstream f(filename.c_str());
	stringstream ss;
	ss << f.rdbuf();
	f.close();
	return ss.str();
}


void add_directory(fs::path directory, std::map<std::string, std::string>& files, int depth = 0, std::string v_path = "/") {
    if (depth > 3) {
        return;
    }
    for (const auto& entry : fs::directory_iterator(directory)) {
        std::string filename = entry.path().filename().string();
        if (entry.is_directory()) {
            add_directory(entry.path(), files, depth + 1, v_path + filename + "/");
        } else if (entry.is_regular_file()) {
            std::string fullPath = entry.path().string();
            files[v_path+filename] = read_file(fullPath);
        }
    }
}


void simple_http::init(){
    add_directory(param_directory, files);
}


void simple_http::tick(){
    auto data = in_rr->data.str();
    auto hdr_end = data.find("\r\n\r\n");
    if(hdr_end != std::string::npos){
        if (data.rfind("GET ", 0) == 0) {
            auto file_name = data.substr(4,data.find(" ",4) - 4);
            if(file_name.size() == 0){
                return;
            }
            if(file_name[file_name.size()-1] == '/'){
                file_name += "index.html";
            }
            auto file = files.find(file_name);
            string rsp = "HTTP/1.1 404 Not Found\r\n\r\n<html><body>404 file not found</body></html>";
            if(file != files.end()){
                rsp = "HTTP/1.1 200 OK\r\n\r\n" + file->second;
            }
            in_rr->send(rsp);
            in_rr->close();
        }
    } // hdr complete
}


void simple_rest::init(){
    default_rr->data.realloc((void*)param_default_rest_data.c_str(),param_default_rest_data.size());
}

void simple_rest::tick(){
    out_rest_rr = default_rr;
 
    auto data = in_rr->data.str();
    auto hdr_end = data.find("\r\n\r\n");
    if(hdr_end != std::string::npos){
        if (data.rfind("POST /"+ param_url, 0) == 0) {
            auto hdr = data.substr(0,hdr_end);
            auto lines = abmt::util::str_split(hdr, "\r\n");
            uint32_t len = 0;
            for(auto& l:lines){
                auto field = abmt::util::str_split(l,": ");
                if(field[0] == "Content-Length"){
                    len = stoi(field[1]);
                }
            }
            if(len + hdr.size() + 4 != in_rr->data.size){
                // incomplete (4 == "\r\n\r\n")
                return;
            }
            string rsp = "HTTP/1.1 200 OK\r\nAccess-Control-Allow-Origin: *\r\n\r\n";
            in_rr->send(rsp);
            out_rest_rr.reset(new http_rest_rr(in_rr));
            out_rest_rr->data.realloc((in_rr->data.data + hdr.size() + 4), len);
        }
    } // hdr complete
}
