/**
 * Author: Hendrik van Arragon, 2023
 * SPDX-License-Identifier: MIT
 */

#ifndef IO_HTTP_H
#define IO_HTTP_H IO_HTTP_H

#include <map>
#include <string>
#include <abmt/io/rcv_rsp.h>

namespace io{

struct http_rest_rr;

/**
 * Simple HTTP-Server
 * This is not made with securty in mind.
 * Don't put it on the internet!
 * @node: auto
 * @raster: auto
 */
struct simple_http{
    abmt::io::rcv_rsp_ptr in_rr;
    
    std::string param_directory = "../html";
    
    std::map<std::string,std::string> files;
    
    void init();
    void tick();
    void error(std::string msg);
};


//@node: auto
struct simple_rest{
    abmt::io::rcv_rsp_ptr in_rr;
    abmt::io::rcv_rsp_ptr out_rest_rr;
    
    std::string param_url = "rest";
    std::string param_default_rest_data = "{}";
    
    std::map<std::string,std::string> files;
    abmt::io::rcv_rsp_ptr default_rr;
    
    
    void init();
    void tick();
};

struct http_rest_rr: public abmt::io::rcv_rsp{
    abmt::io::rcv_rsp_ptr con;
    bool closed=false;

    http_rest_rr(abmt::io::rcv_rsp_ptr c): con(c){
        
    }
    
    virtual void send(void* data, size_t size){
        con->send(data,size);
        close();
	}
	virtual void close(){
	    closed = true;
        con->close();
	}
	
	virtual ~http_rest_rr(){
        if(closed == false){
            abmt::log("Warning: close rest request without response. You sould call send() for every request...");
	        close();
        }
	}
};




} // namespace io

#endif // IO_HTTP_H
