#ifndef IO_TCP_H
#define IO_TCP_H IO_TCP_H

#include <string>
#include <cstdint>
#include <abmt/os.h>
#include <abmt/blob.h>
#include <abmt/time.h>
#include <abmt/io/eio.h>
#include <abmt/io/rcv_rsp.h>

namespace io{
    
struct tcp_rr: public abmt::io::rcv_rsp{
    abmt::io::tcp_ptr con;
    bool new_data = false;
    
    tcp_rr(abmt::io::tcp_ptr c);
    
    virtual void send(void* data, size_t size){
        con->send(data,size);
	}
	virtual void close(){
        con->close();
	}
    
};

//@node: auto
//@raster: source
struct tcp_server_rr{
    abmt::io::rcv_rsp_ptr out_rr;
    std::string param_target = "localhost";
    uint16_t param_port = 12345;
    uint32_t sleep_time_ms = 10;
    
    abmt::io::event_list lst;
    abmt::io::tcp_server_ptr server;
    std::vector<abmt::default_ptr<tcp_rr>> connections;
    
    void init();
    abmt::time poll();
    void final();
};


//@node: auto
//you can also test with "$ nc localhost 12345"
struct tcp_server{
    abmt::blob_shared in_snd_to_all;
    abmt::blob_shared out;
    std::string param_target = "localhost";
    uint16_t param_port = 12345;
    
    abmt::io::event_list lst;
    abmt::io::tcp_server_ptr server;
    std::vector<abmt::io::tcp_ptr> connections;
    
    void init();
    void tick();
};

//@node: auto
struct tcp_client{
    abmt::blob_shared in;
    abmt::blob_shared out;
    std::string param_target = "localhost";
    uint16_t param_port = 12345;
    
    abmt::io::event_list lst;
    abmt::io::tcp_ptr con;
    abmt::io::timer reconnect_timer = {lst,1000};
    
    void init();
    void tick();
    void connect();
};

} // namespace io

#endif // IO_TCP_H
